require 'spec_helper'

describe CapGenerator, type: :generator do
  destination File.expand_path("../../tmp", __FILE__)
  let!(:file) { File.join(destination_root, 'config', 'recipes', 'hoppinger.rb') }

  before do
    prepare_destination
    run_generator
  end

  it 'shuould generate a file called hoppinger.rb' do
    expect(File.exist?(file)).to be(true)
  end

  it 'hoppinger.rb should containt TODO' do
    expect(File.read(file).gsub(/^\s+/, '')).to eq <<-EOT.gsub(/^\s+/, '')
      namespace :db do
        desc 'Backups the db to ~'
        task :backup do
          on roles(:app) do
            deployer_home = "/home/\#{fetch(:user)}"
            db_username, db_password, db_name, host = remote_database_config(fetch(:stage))
            time = Time.now.to_i
            backup_name = "\#{time}-\#{db_name}-\#{fetch(:stage)}.sql"
            execute :mysqldump, "-u \#{db_username} -p'\#{db_password}' \#{db_name} > ~/\#{backup_name}"
            download! "/home/deployer/\#{backup_name}", "\#{backup_name}"
            execute :rm, "\#{deployer_home}/\#{backup_name}"
          end
        end
      end

      namespace :logs do
        desc 'Tails the logs'
        task :tail do
          on roles(:app) do
            execute :tail, "\#{shared_path}/log/\#{fetch(:rails_env)}.log"
          end
        end
      end

      namespace :run do
        desc 'Run rails console'
        task :console do
          on roles(:app) do |host|
            execute_interactively "ruby \#{current_path}/script/rails console \#{fetch(:rails_env)}"
          end
        end

        desc 'Run rake task'
        task :rake do
          on roles(:app) do
            within release_path do
              with rails_env: fetch(:rails_env) do
                execute :rake, 'notes'
              end
            end
          end
        end
      end

      def execute_interactively(command)
        user = fetch(:user)
        port = fetch(:port) || 22
        exec "ssh -l \#{user} \#{host} -p \#{port} -t 'cd \#{deploy_to}/current && \#{command}'"
      end

      def remote_database_config(db)
        remote_config = capture("cat \#{shared_path}/config/database.yml")
        database = YAML::load(remote_config)
        return database["\#{db}"]['username'], database["\#{db}"]['password'], database["\#{db}"]['database'], database["\#{db}"]['host']
      end
    EOT
  end
end